@@ -172,7 +172,7 @@ setMethod("is.na", "Vector", function(x) rep.int(FALSE, length(x)))
     x_len <- length(x)
     x_pslotnames <- parallel_slot_names(x)
     if (!is.character(x_pslotnames)
-     || anyMissing(x_pslotnames)
+     || anyNA(x_pslotnames)
      || anyDuplicated(x_pslotnames)) {
         msg <- c("'parallel_slot_names(x)' must be a character vector ",
                  "with no NAs and no duplicates")
@@ -528,7 +528,7 @@ setReplaceMethod("[", "Vector",
         }
         value <- normalizeSingleBracketReplacementValue(value, x)
         if (is.null(value)) {
-            return(extractROWS(x, complement(nsbs)))
+            return(extractROWS(x, complementNSBS(nsbs)))
         }
         value <- recycleSingleBracketReplacementValue(value, x, nsbs)
         mergeROWS(x, i, value)
@@ -582,7 +582,7 @@ setMethod("replaceROWS", c("Vector", "ANY"),
         ## subscript to use in 'extractROWS(ans, idx)'. By wrapping it inside a
         ## NativeNSBS object, extractROWS() won't waste time checking it or
         ## trying to normalize it.
-        idx <- NativeNSBS(idx, NROW(ans), TRUE, FALSE)
+        idx <- NativeNSBS(idx, NROW(ans))
         ## We assume that extractROWS() works on an object of class 'class(x)'.
         ## For some objects (e.g. Hits), extractROWS() will take care of
         ## validating the returned object.
